/* Fakiir
 *
 * (C) Copyright 2005 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * "$Id: $"
 *
 */

#ifndef __MODEL_H__
#define __MODEL_H__

#include <string>
#include <ext/hash_map>

using namespace std;
using namespace __gnu_cxx; // thanks a bunch to Adrinael from #C++ on freenode

class Object;
class Method;
class Value;
class Property;
class Argument;

struct eqstr {
  bool operator()(const char* s1, const char* s2) const {
    return strcmp(s1, s2) == 0;
  }
};

typedef hash_map<const char*, Object*, hash<const char*>, eqstr > Object_map;
typedef hash_map<const char*, Method*, hash<const char*>, eqstr > Method_map;
typedef hash_map<const char*, Property*, hash<const char*>, eqstr > Property_map;

typedef vector<Argument*> Argument_map;

typedef void (gw_func)(void *obj, Argument_map *args);

#define INT_VALUE     1
#define FLOAT_VALUE   2
#define STRING_VALUE  3


class Object {
 public:
  Object(string n);
  ~Object();
  
  string name;
  
  void init(void *real);
  
  Method_map methods;
  
  Property_map properties;
  
  Object *parent;
  
  Object_map childs;
  
  void *real;
  
};


class Method {
	public:
		Method(string n);
		~Method();

		bool exec();

		string name;

		Object *parent;

		Argument_map args;

		gw_func *gateway;
};

class Value {
	public:
		Value();
		~Value();

		bool parse(char *desc);
		string name;
		
		int type;
		
		bool set(char *arg);

		void *value;
};
		
class Argument: public Value {
	public:
		Argument();
		~Argument();

		int flags;

		Method *method;
};

class Property: public Value {
	public:
		Property();
		~Property();

		Object *parent;
};

#endif
