/* Fakiir
 *
 * (C) Copyright 2005 Denis Rojo <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * "$Id: $"
 *
 */

#include <iostream>
#include <stdio.h>

#include <fakiir-models.h>

FILE *code;



void methodgen(Method_map *methods) {
  Method_map::iterator m_iter;
  Method *m;

  Argument_map::iterator a_iter;
  Argument *a;

  string argstr;
  int c;
  char conv[64];

  fputs("//////////////////////////////////////////\n",code);
  fputs("// Gateway functions for exec() methods\n",code);
  fputs("// add below an include to class header\n",code);
  for( m_iter = methods->begin(); m_iter != methods->end(); ++m_iter) {
    m = (*m_iter).second; // take the data_type out of the pair
    fputs("\nvoid gw_", code);
    fputs(m->name.c_str(), code);
    fputs("(void *obj, Argument_map *args) {\n", code);
    
    // setup argument string
    argstr = "\n(";
    c = 0;

    // fill up arguments
    for(a_iter = m->args.begin(); a_iter != m->args.end(); ++a_iter) {
      a = (*a_iter);

      if     (a->type == INT_VALUE)    argstr += "*(int*)";
      else if(a->type == FLOAT_VALUE)  argstr += "*(float*)";
      else if(a->type == STRING_VALUE) argstr += "(char*)";
      
      argstr += "((*args)[";
      sprintf(conv,"%u",c);
      argstr += conv;
      argstr += "]->value)";
      
    }

    argstr += ")";	
    fprintf(stderr,"DEBUG: gw arg string: %s\n",argstr.c_str());
    // argstr done
    
    // (( method->parent->name *)obj)-> method->name(
    fputs("\t((", code);
    fputs(m->parent->name.c_str(), code);
    fputs("*)obj)->", code);
    fputs(m->name.c_str(), code);
    fputs(argstr.c_str(), code);
    fputs(";\n}\n", code);
    
  }
  
  fputs("/////////// end of gateway functions\n\n",code);
}

void objectgen(Object *o) {
  Method_map::iterator m_iter;
  Method *m;
  
  Property_map::iterator p_iter;
  Property *p;

  Argument_map::iterator a_iter;
  Argument *a;
  
  // obj = new Object("o->name");
  fputs("\nobj = new Object(\"",code);
  fputs(o->name.c_str(),code);
  fputs("\");\n",code);

  // loop thru the methods of the object
  for( m_iter = o->methods.begin(); m_iter != o->methods.end(); ++m_iter) {
    m = (*m_iter).second; // take the data_type out of the pair

    // met = new Method(o->methods.*.name);
    fputs("\nmet = new Method(\"",code);
    fputs(m->name.c_str(),code);
    fputs("\");\n",code);
    
    // assign gateway function to method
    { string tmp = "gw_" + m->name + ";\n";
    fputs("met->gateway = ", code);
    fputs(tmp.c_str(), code);
    }
    
    for(a_iter = m->args.begin(); a_iter != m->args.end(); ++a_iter) {
      a = (*a_iter);
      
      // arg = new Argument();
      fputs("\narg = new Argument();\n",code);

      string tmp;
      tmp = (a->type==INT_VALUE)?"int ":
	(a->type==FLOAT_VALUE)?"float ":
	(a->type==STRING_VALUE)?"char ":
	"void";
      tmp += a->name;

      fputs("arg->parse(\"",code);
      fputs(tmp.c_str(),code);
      fputs("\");\n",code);

      /*
      // arg->name = "a->name"
      fputs("arg->name = \"",code);
      fputs(a->name.c_str(),code);
      fputs("\";\n",code);
      
      // arg->type = "a->type"
      fputs("arg->type = ",code);
      fputs(	(a->type==INT_VALUE)?"INT_VALUE":
		(a->type==FLOAT_VALUE)?"FLOAT_VALUE":
		(a->type==STRING_VALUE)?"STRING_VALUE":"0",
		code);
      fputs(";\n",code);
      
      // arg->format = "a->format"
      fputs("arg->format = \"",code);
      fputs(a->format.c_str(),code);
      fputs("\";\n",code);
      */

      // met->args.append( arg );
      fputs("met->args.push_back( arg );\n",code);
      
      // arg->method = met;
      fputs("arg->method = met;\n",code);
      
    }
    
    // obj->methods.append( met );
    fputs("\nobj->methods[ met->name.c_str() ] =  met;\n",code);
    
    // met->parent = obj;
    fputs("met->parent = obj;\n",code);
    
  }

  for(p_iter = o->properties.begin(); p_iter != o->properties.end(); ++p_iter) {
    p = (*p_iter).second; // take out the data_type from the pair

    // prop = new Property();
    fputs("\nprop = new Property();\n",code);
    
    // prop->name = "p->name";
    fputs("prop->name = \"",code);
    fputs(p->name.c_str(),code);
    fputs("\";\n",code);
    
    // prop->type = "p->type";
    fputs("prop->type = ",code);
    fputs(	(p->type==INT_VALUE)?"INT_VALUE":
		(p->type==FLOAT_VALUE)?"FLOAT_VALUE":
		(p->type==STRING_VALUE)?"STRING_VALUE":"0",
		code);
    fputs(";\n",code);
    
    // prop->format = "p->format"
    fputs("prop->format = \"",code);
    fputs(p->format.c_str(),code);
    fputs("\";\n",code);
    
    // obj->properties.append( prop );
    fputs("obj->properties[ prop->name.c_str() ] =  prop;\n",code);
    
    // prop->parent = obj;
    fputs("prop->parent = obj;\n",code);
    
  }
  
}



////////////////////////////////////////////////////////
// generates the code to reproduce a tree anywhere
void treegen(Object_map *symbols, char *filename) {
  Object_map::iterator o_iter;
  Object *o;

  Object_map::iterator child_iter;
  Object *child;

  code = fopen("fakiir.cpp","w"); 
  if(!code) {
    fprintf(stderr,"can't create file fakiir.cpp\n");
    return; }
  
  fputs("#include <",code);
  fputs(filename,code);
  fputs(">\n",code);
  fputs("#include <fakiir-models.h>\n\n",code);
  
  
  fputs("Object_map fakiir_symbols;\n",code);
  
  // loop thru objects and use methodgen to generate callbacks
  for( o_iter = symbols->begin(); o_iter != symbols->end(); ++o_iter) {
    o = (*o_iter).second;
    methodgen( &o->methods );
  }

  fputs("void fakiir_init() {\n",code);
  fputs("Object *obj;\n",code);
  fputs("Method *met;\n",code);
  fputs("Property *prop;\n",code);
  fputs("Argument *arg;\n\n",code);
  
  for( o_iter = symbols->begin(); o_iter != symbols->end(); ++o_iter) {
    o = (*o_iter).second;
    fputs("///////////////////////////\n",code);
    fputs("// instance of ",code);
    fputs(o->name.c_str(),code);
    fputs("\n\n",code);
    
    objectgen(o);
    
    for(child_iter = o->childs.begin(); child_iter != o->childs.end(); ++child_iter) {
      child = (*child_iter).second; // data_type in the pair

      fputs("\t{\nObject *parent = obj;\n",code);
      objectgen( child );
      fputs("\nparent->childs[ obj->name.c_str() ] = obj;\n",code);
      fputs("obj = parent;\n\t}\n",code);

    }
    
    fputs("\nfakiir_symbols[ obj->name.c_str() ] = obj;\n",code);
    fputs("/////////////////////\n\n",code);
    
    
  }
  fputs("\n}\n",code);
  fclose(code);
}

